@@ -2,10 +2,10 @@
 """Basic classes for running Genetic Algorithms.
 """
 __author__ = "Justin Hocking"
-__copyright__ = "Copyright 2020, Zipfian Science"
+__copyright__ = "Copyright 2021, Zipfian Science"
 __credits__ = []
 __license__ = ""
-__version__ = "0.0.1"
+__version__ = "0.1.0"
 __maintainer__ = "Justin Hocking"
 __email__ = "justin.hocking@zipfian.science"
 __status__ = "Development"
@@ -29,7 +29,7 @@ class Gene:
     Args:
         name (str): Gene name. The gene name also acts as a compatibility reference.
         value (Any): The value, could be any type.
-        randomise_function (Callable): A function to randomise the gene, taking the min and max as input with signature ``func(self)``.
+        randomise_function (Callable): A function to randomise the gene, taking the gene (`self`) as input with signature ``func(self)``.
         gene_max (Any, numeric type): Max value for random number generator (default = None).
         gene_min (Any, numeric type): Min value for random number generator (default = None).
         mu (Any, numeric type): Mean value of distribution to sample from (default = 0).
@@ -108,10 +108,27 @@ class Chromosome:
     """
     A class that encapsulates an ordered sequence of Gene objects.
 
+    Note:
+        gene_verify_func should take the gene, index of gene, and chromosome (`self`) as parameters.
+
     Args:
         genes (list): list of initialised Gene objects.
         gene_verify_func (Callable): A function to verify gene compatibility `func(gene,loc,chromosome)` (default = None).
     """
+
+    def __default_gene_verify_func(self, gene, loc, chromosome):
+        """
+        Needed to make objects pickle-able.
+        Args:
+            gene (Gene): Gene being inserted.
+            loc (int): Index of gene.
+            chromosome (Chromosome): The given "self".
+
+        Returns:
+            bool: Whether gene insertion is allowed or not.
+        """
+        return True
+
     def __init__(self, genes: list = None, gene_verify_func : Callable = None):
         if genes:
             self.genes = genes
@@ -121,7 +138,7 @@ class Chromosome:
         if gene_verify_func:
             self.gene_verify_func = gene_verify_func
         else:
-            self.gene_verify_func = lambda gene, loc, chromosome: True
+            self.gene_verify_func = self.__default_gene_verify_func
 
     def append(self, gene: Gene):
         """
@@ -135,16 +152,22 @@ class Chromosome:
             raise GeneticAlgorithmError(message="Added gene did not pass compatibility tests!")
         self.genes.append(gene)
 
-    def __setitem__(self, index: int, gene: Gene):
-        assert isinstance(gene, Gene), 'Must be Gene type!'
-        assert index < len(self.genes), 'Index Out of bounds!'
+    def __setitem__(self, index, gene):
+        if isinstance(index, slice):
+            assert index.start < len(self.genes), 'Index Out of bounds!'
+        else:
+            assert isinstance(gene, Gene), 'Must be Gene type!'
+            assert index < len(self.genes), 'Index Out of bounds!'
 
         if not self.gene_verify_func(gene=gene, loc=index, chromosome=self):
             raise GeneticAlgorithmError("Index set gene did not pass compatibility tests!")
         self.genes[index] = gene
 
-    def __getitem__(self, index: int) -> Gene:
-        assert index < len(self.genes), 'Index Out of bounds!'
+    def __getitem__(self, index) -> Gene:
+        if isinstance(index, slice):
+            assert index.start < len(self.genes), 'Index Out of bounds!'
+        else:
+            assert index < len(self.genes), 'Index Out of bounds!'
 
         return self.genes[index]
 
@@ -176,6 +199,13 @@ class Individual:
         name (str): Name for keeping track of lineage (default = None).
         chromosome (Chromosome): A Chromosome object, initialised (default = None).
         species_type (str) : A unique string to identify the species type, for preventing cross polluting (default = None).
+
+    Attributes:
+        fitness (Numeric): The fitness score after evaluation.
+        age (int): How many generations was the individual alive.
+        genetic_code (str): String representation of Chromosome.
+        history (list): List of dicts of every evaluation.
+        parents (list): List of strings of parent names.
     """
 
     def __init__(self, fitness_function : Callable, name : str = None, chromosome: Chromosome = None, species_type : str = None):
@@ -264,8 +294,18 @@ class Individual:
             numeric: Fitness value.
         """
         self.fitness = self.fitness_function(chromosome=self.chromosome, island=island, **params)
-        self.history.append({"name" : self.name, "age" : self.age, "fitness" : self.fitness,
-                             "chromosome" : self.unique_genetic_code, "parents" : self.parents})
+
+        stamp = { "name": self.name,
+                  "age": self.age,
+                  "fitness": self.fitness,
+                  "chromosome": self.unique_genetic_code(),
+                  "parents": self.parents,
+         }
+
+        if island:
+            stamp["island_generation" ] = island.generation_count
+
+        self.history.append(stamp)
         return self.fitness
 
     def unique_genetic_code(self) -> str:
@@ -286,14 +326,7 @@ class Individual:
 class Island:
     """
     A simple Island to perform a Genetic Algorithm. By default the selection, mutation, crossover, and probability functions
-    default to the classic functions:
-    - `natural_selection.ga.selection.selection_function_classic`
-    - `natural_selection.ga.selection.selection_function_parents_classic`
-    - `natural_selection.ga.crossover.crossover_function_classic`
-    - `natural_selection.ga.mutation.mutation_function_classic`
-    - `natural_selection.ga.prob_functions.crossover_prob_function_classic`
-    - `natural_selection.ga.prob_functions.mutation_prob_function_classic`
-    - `natural_selection.ga.helper_functions.clone_function_classic`
+    default to the classic functions.
 
     Args:
         function_params (dict): The parameters for the fitness function.
@@ -304,9 +337,19 @@ class Island:
         crossover_prob_function (Callable): Random probability function for crossover (default = None).
         mutation_prob_function (Callable): Random probability function for mutation (default = None).
         clone_function (Callable): Function for cloning (default = None).
+        random_seed (int): Random seed for random and Numpy generators, set to None for no seed (default = 42).
         verbose (bool): Print all information (default = None).
         logging_function (Callable): Function for custom message logging, such as server logging (default = None).
         force_genetic_diversity (bool): Only add new offspring to the population if they have a unique chromosome (default = True).
+
+    Attributes:
+        unique_genome (list): List of unique chromosomes.
+        generation_info (list): List of dicts detailing info for every generation.
+        population (list): The full population of members.
+        elites (list): All elites selected during the run.
+        mutants (list): All mutants created during the run.
+        children (list): All children created during the run.
+        generation_count (int): The current generation number.
     """
 
     def __init__(self, function_params : dict,
@@ -317,6 +360,7 @@ class Island:
                  crossover_prob_function : Callable = crossover_prob_function_classic,
                  mutation_prob_function : Callable = mutation_prob_function_classic,
                  clone_function : Callable = clone_classic,
+                 random_seed: int = 42,
                  verbose : bool = True,
                  logging_function : Callable = None,
                  force_genetic_diversity : bool = True):
@@ -338,13 +382,19 @@ class Island:
         self.force_genetic_diversity = force_genetic_diversity
 
         self.verbose = verbose
-
+        self.random_seed = random_seed
         self.elites = list()
         self.mutants = list()
         self.children = list()
         self.species_type = "DEFAULT_SPECIES"
         self.generation_count = 0
 
+        # Set python random seed, as well as Numpy seed.
+        if random_seed:
+            random.seed(random_seed)
+            from numpy.random import seed as np_seed
+            np_seed(random_seed)
+
     def create_gene(self,
                  name : str,
                  value : Any,
@@ -414,7 +464,6 @@ class Island:
 
     def initialise(self, adam : Individual,
                population_size: int = 8,
-               random_seed : int = 42,
                initialisation_function : Callable = initialise_population_random,
                initialisation_params : dict = {},
                evaluate_population : bool = True
@@ -425,13 +474,10 @@ class Island:
         Args:
             adam (Individual): Individual to clone from.
             population_size (int): Size of population.
-            random_seed (int): Random seed (default = 42).
             initialisation_function (Callable): A function for randomly creating new individuals from the given adam.
             initialisation_params (dict): Custom params for custom initialisation functions.
             evaluate_population (bool): Evaluate the newly created population (default = True).
         """
-        random.seed(random_seed)
-
         self.species_type = adam.species_type
 
         self.initialise = initialisation_function
@@ -440,7 +486,7 @@ class Island:
 
         if evaluate_population:
             for popitem in self.population:
-                popitem.evaluate(self.function_params)
+                popitem.evaluate(self.function_params, island=self)
                 self.unique_genome.append(popitem.unique_genetic_code())
 
     def import_migrants(self, migrants : list,
@@ -464,7 +510,7 @@ class Island:
             if force_genetic_diversity:
                 if not i.unique_genetic_code() in self.unique_genome:
                     if i.fitness is None or reset_fitness:
-                        i.evaluate(self.function_params)
+                        i.evaluate(self.function_params, island=self)
                     self.population.append(i)
                     self.unique_genome.append(i.unique_genetic_code())
             else:
@@ -528,14 +574,17 @@ class Island:
         else:
             _survivor_selection_params = {}
 
+        def _default_criterion_function(island):
+            return island.generation_count < starting_generation + n_generations - 1
+
         if criterion_function:
             g_func = criterion_function
         else:
-            g_func = lambda island: island.generation_count >= starting_generation + n_generations - 1
+            g_func = _default_criterion_function
             criterion_params = {}
 
         g = starting_generation
-        while not g_func(island=self, **criterion_params):
+        while g_func(island=self, **criterion_params):
             self._verbose_logging('Generation {} started'.format(g))
 
             self.__evolutionary_engine(g=g,
