@@ -12,7 +12,7 @@ __status__ = "Development"
 
 def selection_parents_two(individuals : list, island=None):
     """
-    Simple function to select two parents at a time, sequentially.
+    Simple function to select two parents at a time, sequentially. Parental selection always yields.
 
     Args:
         individuals (list): A list of Individuals, specifically the selected "elites".
@@ -22,4 +22,10 @@ def selection_parents_two(individuals : list, island=None):
         list: Containing the two individuals selected for crossover.
     """
     for parent_1, parent_2 in zip(individuals[::2], individuals[1::2]):
-        yield [parent_1, parent_2]
\ No newline at end of file
+        yield [parent_1, parent_2]
+
+def selection_parents_two_shuffled(individuals : list, island=None):
+    raise NotImplementedError("Future implementation")
+
+def selection_parents_n_gram(individuals : list, n : int = 2,island=None):
+    raise NotImplementedError("Future implementation")
\ No newline at end of file
