@@ -14,6 +14,12 @@ def clone_classic(individuals : list, island=None):
     return copy.deepcopy(individuals)
 
 class GeneticAlgorithmError(Exception):
+    """
+    Encapsulating graceful exception handling during evolutionary runs.
+    Args:
+        message (str): Message to print.
+        exit (bool): Whether to hard exit the process or not (default = False).
+    """
 
     def __init__(self, message : str, exit : bool = False):
         self.message = message
